package com.gic.cloud.data.hook.service;

import cn.medubi.client.utils.LogPak;
import com.alibaba.fastjson.JSON;
import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigService;
import com.gic.commons.util.DingtalkMessageUtil;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.List;

/** Hive 辅助工具类
 * @author Sam
 */
@Component
public class HiveHelper implements ApplicationContextAware {

	/** 日志类 */
	private static LogPak log = new LogPak(HiveHelper.class);

	/** 内置数据源 */
	private static DataSource source = null;

	private static DataSource downloadSource = null;

	private static DataSource balaDownloadSource = null;

	private static DataSource balaSearchSource = null;

	private static DataSource bigDataDownloadSource = null;

	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		log.debug("setApplicationContext", "准备初始化 Hive 连接池");
		source = (DataSource)applicationContext.getBean("hiveSource");
		downloadSource = (DataSource)applicationContext.getBean("downloadHiveSource");
		balaDownloadSource = (DataSource)applicationContext.getBean("balaDownloadHiveSource");
		balaSearchSource = (DataSource)applicationContext.getBean("balaSearchHiveSource");
		bigDataDownloadSource = (DataSource)applicationContext.getBean("bigTaskdownloadHiveSource");
	}

	/** 获取 Hive 数据源
	 * @return
	 */
//	public static DataSource getDataSource() {
//		return source;
//	}

	/** 获取一个 Hive 的连接
	 * @return
	 */
	public static Connection getHiveConnection() {
		Connection conn = null;
		try {
			conn = source.getConnection();
			return conn;
		} catch (Exception ex) {
			ex.printStackTrace();
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
			return null;
		} // TRY CATCH OVER
	}

	public static Connection getDownloadHiveConnection() {
		Connection conn = null;
		try {
			conn = downloadSource.getConnection();
			return conn;
		} catch (Exception ex) {
			DingtalkMessageUtil.sendAlertMessage("获取连接告警：获取连接超时", "https://oapi.dingtalk.com/robot/send?access_token=157ad00c2b6491f2f0aac1d89121e4bd2d82d9d33cad0596b88dacfdc12fe455");
			ex.printStackTrace();
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
			throw new RuntimeException("获取连接异常");
		} // TRY CATCH OVER
	}


	public static Connection getBalaDownloadHiveConnection() {
		Connection conn = null;
		try {
			conn = balaDownloadSource.getConnection();
			return conn;
		} catch (Exception ex) {
			DingtalkMessageUtil.sendAlertMessage("获取bala下载连接告警：获取连接超时", "https://oapi.dingtalk.com/robot/send?access_token=157ad00c2b6491f2f0aac1d89121e4bd2d82d9d33cad0596b88dacfdc12fe455");
			ex.printStackTrace();
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
			throw new RuntimeException("获取连接异常");
		} // TRY CATCH OVER
	}

	public static Connection getBalaSearchHiveConnection() {
		Connection conn = null;
		try {
			conn = balaSearchSource.getConnection();
			return conn;
		} catch (Exception ex) {
			ex.printStackTrace();
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
			return null;
		} // TRY CATCH OVER
	}

	public static Connection getBigDataDownloadHiveConnection() {
		Connection conn = null;
		try {
			conn = bigDataDownloadSource.getConnection();
			return conn;
		} catch (Exception ex) {
			DingtalkMessageUtil.sendAlertMessage("获取连接告警：获取连接超时", "https://oapi.dingtalk.com/robot/send?access_token=157ad00c2b6491f2f0aac1d89121e4bd2d82d9d33cad0596b88dacfdc12fe455");
			ex.printStackTrace();
			if (conn != null) {
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
			throw new RuntimeException("获取连接异常");
		} // TRY CATCH OVER
	}

	public static Connection getSearchHiveConnection(List<String> enterpriseIds){
		Config config = ConfigService.getConfig("application");
		String customRouterEnterpriseId = config.getProperty("customRouterEnterpriseId", "");
		if(enterpriseIds.contains(customRouterEnterpriseId)){
			return HiveHelper.getBalaSearchHiveConnection();
		} else {
			return HiveHelper.getHiveConnection();
		}
	}

}
